package net.pricefx.domain;

import java.time.LocalDateTime;
import java.util.*;

import net.pricefx.common.api.FieldFormatType;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;


public class LookupTable extends TimedObjectWithStatus {

	public enum LookupTableType {
		SIMPLE, RANGE, MATRIX, SIMPLE_INT_KEY, SIMPLE_REAL_KEY, SIMPLE_DATE_KEY, JSON;
		
		private static final Map<String,LookupTableType> lookup = new HashMap<String,LookupTableType>();

		static {
		    for(LookupTableType ct : EnumSet.allOf(LookupTableType.class)){
		         lookup.put(ct.name(), ct);
		    }
		}

	    public static LookupTableType get(String code) {
	        if(code == null) return null;
	       	return lookup.get(code.toUpperCase());
	    }
	}


	public enum LookupTableValueType {
		STRING, INT, REAL, MATRIX, DATE, MATRIX2, MATRIX3, MATRIX4, MATRIX5, MATRIX6, JSON, JSON2;

		private static final Map<String,LookupTableValueType> lookup = new HashMap<String,LookupTableValueType>();

		static {
		    for(LookupTableValueType ct : EnumSet.allOf(LookupTableValueType.class)){
		         lookup.put(ct.name(), ct);
		    }
		}

	    public static LookupTableValueType get(String code) {
	        if(code == null) return null;
	       	return lookup.get(code.toUpperCase());
	    }
	}

	private static final String[] RANGE_KEYS = {"lowerBound", "upperBound"};

	private LookupTableType type;

	private LookupTableValueType valueType;

	private Long nodeId;

	private String userGroupEdit;

	private String userGroupViewDetails;

	private Boolean hideWarnings;

    private FieldFormatType formatType;

	private Boolean isPlasma;

	private String lastUpdateByName;

	private String createdByName;

	public LookupTable(Partition p,String uniqueName, String label) {
		this();
		this.setPartition(p);
		this.setUniqueName(uniqueName);
		this.setLabel(label);
	}

	public LookupTable() {
		super();
	}

	public LookupTableValueType getValueType() {
		return valueType;
	}

	public void setValueType(LookupTableValueType valueType) {
		this.valueType = valueType;
	}

	public LookupTableType getType() {
		return type;
	}

	public void setType(LookupTableType type) {
		this.type = type;
	}

	public boolean getHideWarnings() {
		return hideWarnings == null ? false : hideWarnings.booleanValue();
	}

	public void setHideWarnings(Boolean hideWarnings) {
		this.hideWarnings = hideWarnings;
	}

	public FieldFormatType getFormatType() {
		return formatType;
	}

	public void setFormatType(FieldFormatType formatType) {
		this.formatType = formatType;
	}

	// explicitly overriding to get id into response
	@JsonProperty(value="id")
	public Long getLookupTableId() {
		return getId();
	}

	@JsonProperty(value="id")
	public void setLookupTableIdShouldBeIgnored(Long id) {
		// just to satisfy deserialization
	}

	public Long getNodeId() {
		return nodeId;
	}

	public void setNodeId(Number nodeId) {
		if (nodeId instanceof Long || nodeId instanceof Integer
				|| nodeId instanceof Short || nodeId instanceof Byte) {
			this.nodeId = nodeId.longValue();
		}
	}

	public String getUserGroupEdit() {
		return userGroupEdit;
	}

	public void setUserGroupEdit(String userGroup) {
		this.userGroupEdit = userGroup;
	}

	public String getUserGroupViewDetails() {
		return userGroupViewDetails;
	}

	public void setUserGroupViewDetails(String userGroup) {
		this.userGroupViewDetails = userGroup;
	}

	public String getLastUpdateByName() {
		return lastUpdateByName;
	}

	public void setLastUpdateByName(String lastUpdateByName) {
		this.lastUpdateByName = lastUpdateByName;
	}

	public String getCreatedByName() {
		return createdByName;
	}

	public void setCreatedByName(String createdByName) {
		this.createdByName = createdByName;
	}


	public static boolean isMatrixLookupTableValueClass(final Class<?> clazz){
		return MatrixLookupTableValue.class.equals(clazz) || MatrixLookupTableValue2Key.class.equals(clazz)	||
				MatrixLookupTableValue3Key.class.equals(clazz) || MatrixLookupTableValue4Key.class.equals(clazz) ||
				MatrixLookupTableValue5Key.class.equals(clazz) || MatrixLookupTableValue6Key.class.equals(clazz);
	}

	public int getNumberOfKeyFields() {
		if (valueType != null) {
			// RANGE tables have (lowerBound, upperBound) pair as composite key
			if (LookupTableType.RANGE.equals(type)) {
				return RANGE_KEYS.length;
			}
			switch (valueType) {
				case JSON2:
				case MATRIX2:
					return 2;
				case MATRIX3:
					return 3;
				case MATRIX4:
					return 4;
				case MATRIX5:
					return 5;
				case MATRIX6:
					return 6;
				default:
					// do nothing
			}
		}
		return 1;
	}

	@SuppressWarnings("unused")
	private void setNumberOfKeyFields(int n){
		// for json deserialization only
	}

	@Override
	public boolean doHardDelete() {
		return true;
	}

	@JsonIgnore
	public Map<String,Object> getBusinessKey() {
		Map<String,Object> busKey = new HashMap<>();
		busKey.put("uniqueName", getUniqueName());
//		busKey.put("validAfter", setToMidnight(getValidAfter()));
		busKey.put("simulationSet", getSimulationSet());
		return busKey;
	}

	@JsonProperty("isPlasma")
	public boolean isPlasma() {
		return isPlasma==null ? false : isPlasma.booleanValue();
	}

	@JsonProperty("isPlasma")
	public void setPlasma(Boolean isPlasma) {
		this.isPlasma = isPlasma;
	}


	public boolean onPreProcessing(Map<String, Object> values, LocalDateTime targetDate) {
		return true;
	}

}
