package net.pricefx.formulaengine;

import com.googlecode.genericdao.search.Filter;
import net.pricefx.domain.DMModel;
import net.pricefx.formulaengine.ModelFormulaContext.TreeNode;
import net.pricefx.lmformulaengine.LmFormulaEngineContext;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A ModelEvaluator exposes a previously prepared/trained/calculated model to non-PO modules (Groovy logics).
 * The idea is that the internals / implementation details of the model are shielded, and that
 * only the behaviour intended to be exposed is accessible to the outside world.
 * <p>Specifically, the {@link #evaluate(Map)} method executes the model's logic in the <code>evaluation</code> calculation context.
 * Depending on the ModelType definition, this could be a dedicated evaluation logic, or elements embedded in the model's main logic.
 * In addition to evaluation, there is also a number of helper methods, which simplify getting objects related to (and usually generated by) a model.
 *
 * <p>
 * Example:  The standard Segmentation's evaluation logic determines the segment for a given set of (segmentation attribute, value) pairs, and
 * returns the PolicyRecord for that segment, valid on a given date.  The record contains the guidance and key metrics for that segment that
 * an external application can translate to, for example, a target price for a product quoted to a customer.
 * The evaluation logic is seen as implementation detail to the model.  But the standard way of calling it, by means of a <code>ModelEvaluator</code>
 * is not, as it represents the contract between the model (or ModelType more generally) and users (clients) of the model.
 * <p><pre><code>
 * def eval = api.modelEvaluator(modelUniqueName)
 * def levels = eval.params["Segmentation Levels"]
 * def input = levels.collectEntries{ api.stringUserEntry(it) }
 * def record = eval.evaluate(input)
 * return record
 * </code></pre></p>
 */
public class ModelEvaluator {

    private final DMModel model;

    public static final String RESULTS = "results";
    public static final String ERRORS = "errors";


    public ModelEvaluator(DMModel model, LmFormulaEngineContext context) {
        this.model = model;
    }

    /**
     * Executes a model's evaluation logic, taking a map of input parameters.  The expected inputs, as well as the returned result,
     * are entirely dependent on the model, more specifically the ModelType of the model.
     * @param inputs    Map of input param (name,value) pairs
     * @return  Map containing one or two entries.  One entry that is always there has the key <code>"results"</code>.  Its value is the evaluation logic's
     * calculation results as a map of (element name, result) pairs.
     * If there was any error evaluating the logic, the result map will also contain an entry with the key <code>"errors"</code>, and as value the error messages
     * (in {@link net.pricefx.domain.ValidationVisitor format).
     */
    public Map<String,Object> evaluate(Map<String,Object> inputs) {
        return null;
    }

    /**
     * Retrieves the root node of the tree with the given name, if it exists (i.e. was created by the model).
     * @param treeName  Name of the tree, as specified in e.g. {@link ModelFormulaContext#addTree(String, Collection)}
     * @return  The root node of the tree
     */
    public TreeNode getTree(final String treeName) {
        return null;
    }

    /**
     * Determines the node in a model tree that complies to the given input, starting from a given TreeNode.
     * Usually the starting TreeNode is the root node of the tree, but this is not mandatory.
     * Each node in the tree defines a Filter.  When the filters of the starting node's children do not exclude the given input,
     * the evaluation is passed on to their descendants, where again the input is tested against those nodes' filter criteria.
     * When the traversal reaches a dead-end, the output of the last successfully matched node is returned.
     * The tree is expected to be constructed in such a way that the result is unambivalent, but this is the responsibility of the model.
     * <p>
     *     Example:  In the below tree, the root node has 2 children, one with the filter <code>Filter.equal("Region", "EMEA")</code>
     *     and one with <code>Filter.equal("Region", "APAC")</code>.
     *     The former has itself 1 child with the filter <code>Filter.equal("Channel", "Retail")</code>, the latter has 2 with filters
     *     <code>Filter.equal("Channel", "Retail")</code> and <code>Filter.equal("Channel", "Industry")</code>.
     *     When given an input <code>[Region:"APAC", Channel:"Retail"]</code>, the root node's first child is rejected
     *     but the second child's filter is passed (<code>Region == "APAC"</code>).  The evaluation is then passed to that node's first child,
     *     where it also passes (<code>Channel == "Retail"</code>), and given that this is a leave node, its output is returned up the call stack.
     *     Note that in the Segmentation model, a node's output is the name of the segment represented by that tree node (e.g. 'S000004').
     * <p><pre>
     * Segments
     *      |
     * 	 + EMEA
     * 	 |    |
     * 	 |    + Retail
     * 	 |
     * 	 + APAC
     * 	      |
     * 		  + Retail
     * 		  |
     * 		  + Industry
     * </pre></p>
     * </p>
     * @param tree  The node from which to start the evaluation
     * @param input Map of input param (name,value) pairs
     * @return  The value of the output field of the last matched FilterNode
     */
    public String evaluateTree(final TreeNode tree, final Map<String,Object> input) {
        return null;
    }

    /**
     * Equivalent to {@link ModelFormulaContext#findRecords(Filter, String...)}.
     * @param filter    The criteria the records needs toad here to
     * @param sortBy    Optional sort fields
     * @return  A list of map representations of the records, empty if none matched the criteria.
     */
    public List<Map<String,Object>> findRecords(Filter filter, String... sortBy) {
        return null;
    }

    /**
     * Equivalent to {@link ModelFormulaContext#getParams()}.
     * @return  A map of param (name, value) pairs
     */
    public Map<String,Object> getParams() {
        return null;
    }

    /**
     * Equivalent to {@link ModelFormulaContext#getTable(String)}.
     * @param name
     * @return  The Table object with the given tableName if it exists, or null if not.
     */
    public DatamartContext.Table getTable(final String name) {
        return null;
    }

    public PMMLEvaluator pmmlEvaluator(String attachmentName) {
        return null;
    }

    public interface PMMLEvaluator {
        Map<String, ?> evaluate(Map<String, ?> inputs);
    }
}
