package net.pricefx.formulaengine.optimization;

import net.pricefx.formulaengine.DatamartContext;
import net.pricefx.formulaengine.scripting.PublicGroovyAPI;
import net.pricefx.formulaengine.scripting.RContext;

import java.util.Map;

public interface ModelFormulaContextReadOnly {
    /**
     * @return The unique name of this model
     */
    String uniqueName();

    /**
     * @return The label of this model
     */
    String label();

    /**
     * This gives access to the previous steps saved inputs.
     * <p>
     * Those inputs are organized per step and per tab in the step.
     * <p>
     * From a given step, it is not possible to access its inputs (for this
     * {@link PublicGroovyAPI#input(String)}
     * or the {@code input} binding can be used) or the next step's inputs.
     *
     * @param stepName The name of the step as defined in the model class
     * @param tabName  The name of the tab as defined in the model class
     * @return The unmodifiable inputs of the tab in the step
     */
    Map<String, Object> inputs(String stepName, String tabName);

    /**
     * This method gives access to the results of calculations of this Model object, that is:
     * <ul>
     *     <li>Formula Calculations that produce a result per element.</li>
     *     <li>Job Trigger Calculations that may produce an arbitrary map of results, depending on the image used.</li>
     * </ul>
     * <p>
     * The name of the calculation is the one specified in the Model Class or for Job Trigger Calculations, a name
     * of the form {@code calculationName:jobName} (see {@link ModelFormulaContextReadWrite#startJobTriggerCalculation(String, String, String, String)}).
     * <p>
     * Those outputs are organized per step and per calculation name in the step.
     * <p>
     * From a given step, it is not possible to access the next steps' calculations' outputs.
     * <p>
     * From a given calculation in a sequence, it is not possible to access the next calculations' outputs.
     *
     * @param stepName        The name of the step as defined in the model class
     * @param calculationName The name of the calculation as defined above
     * @return The unmodifiable ordered results of the calculation
     */
    Map<String, Object> outputs(String stepName, String calculationName);

    /**
     * @param name The table name in this model
     * @return The existing model table or {@code null} if it doesn't exist
     */
    DatamartContext.Table table(String name);

    /**
     * @return a context for read operations on the lookup tables of this model
     */
    ModelLookupTablesContextReadOnly lookupTablesContext();

    /**
     * @return a context for read operations on the filter trees of this model
     */
    ModelFilterTreesContextReadOnly filterTreesContext();


    /**
     * @return a context for evaluation of pmml models attached to this model
     */
    ModelPMMLContext pmmlContext();

    /**
     * @return a context for evaluation of arbitrary R code
     */
    RContext rContext();
}
