package net.pricefx.logic.core;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.introspect.Annotated;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.impl.SimpleBeanPropertyFilter;
import com.fasterxml.jackson.databind.ser.impl.SimpleFilterProvider;
import com.intellij.openapi.vfs.VirtualFile;
import net.pricefx.pckg.transform.*;
import net.pricefx.pckg.transform.descriptor.TypeDescriptor;
import net.pricefx.restapiclient.model.BaseMap;
import org.jetbrains.annotations.NotNull;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import static net.pricefx.common.api.PriceFxInterface.FIELD_CREATEDATE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_FE_FORMULAEXPRESSION;
import static net.pricefx.common.api.PriceFxInterface.FIELD_LASTUPDATEBYNAME;
import static net.pricefx.common.api.PriceFxInterface.FIELD_LASTUPDATEDATE;
import static net.pricefx.common.api.PriceFxInterface.FIELD_PL_LAST_UPDATE_BY;
import static net.pricefx.common.api.PriceFxInterface.FIELD_TYPEDID;
import static net.pricefx.common.api.PriceFxInterface.FIELD_VERSION;

/**
 * Various helper methods.
 */
public class Utils {

    private Utils() {
    }

    private static final Set<String> IGNORED_ATTRIBUTES = new HashSet<>();

    static {
        IGNORED_ATTRIBUTES.add(FIELD_FE_FORMULAEXPRESSION);
        IGNORED_ATTRIBUTES.add(FIELD_LASTUPDATEDATE);
        IGNORED_ATTRIBUTES.add(FIELD_PL_LAST_UPDATE_BY);
        IGNORED_ATTRIBUTES.add(FIELD_LASTUPDATEBYNAME);
        IGNORED_ATTRIBUTES.add(FIELD_TYPEDID);
        IGNORED_ATTRIBUTES.add(FIELD_VERSION);
        IGNORED_ATTRIBUTES.add(FIELD_CREATEDATE);
        IGNORED_ATTRIBUTES.add("createdBy");
        IGNORED_ATTRIBUTES.add("errors");
        IGNORED_ATTRIBUTES.add("diskFolder");
    }

    /**
     * User by configurators, form UI components. Compared to Util.getObjectMapper(), this doesn't do indentation.
     *
     * @return
     */
    public static ObjectMapper getObjectMapper() {
        return new ObjectMapper()
                // BaseMap are needed by some form components
                .registerModule(new SimpleModule().addAbstractTypeMapping(Map.class, BaseMap.class))
                // null values are omitted in PFX API in order to reduce the traffic
                .setSerializationInclusion(JsonInclude.Include.NON_NULL)
                // to keep the fields in persisted files sorted
                .enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY)
                .enable(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)
                // PFX API uses "2020-08-25T11:35:49.163+0000" date format instead of a numeric timestamp 1598355307951
                .disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS)
                // do not crash for future PFX versions; can be removed after model converted to BaseMap instead of Object
                .disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES)
                .enable(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL)
                // decimals and long are needed for the ContextParameter value change comparison
                .enable(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS)
                .enable(DeserializationFeature.USE_LONG_FOR_INTS)
                .enable(SerializationFeature.INDENT_OUTPUT);
    }

    /**
     * Keeps empty maps as null. This is used to write params.json and results.json
     *
     * @return
     */
    public static ObjectMapper getObjectMapperWithEmptyMaps() {
        return new ObjectMapper()
                // do not crash for future PFX versions
                .disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES)
                // to keep the fields in persisted files sorted
                .enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY)
                .enable(SerializationFeature.ORDER_MAP_ENTRIES_BY_KEYS)
                // indentation in files
                .enable(SerializationFeature.INDENT_OUTPUT);
    }

    public static String serializeToJSONAsString(Object object) throws JsonProcessingException {
        ObjectMapper objectMapper = getObjectMapperWithEmptyMaps();
        SimpleBeanPropertyFilter filter = SimpleBeanPropertyFilter.serializeAllExcept(IGNORED_ATTRIBUTES);
        SimpleFilterProvider filterProvider = new SimpleFilterProvider()
                .addFilter(FilterOutIgnoredFieldsInstrospector.FILTER_ID, filter);

        objectMapper = objectMapper.setAnnotationIntrospector(new FilterOutIgnoredFieldsInstrospector());
        return objectMapper.writer(filterProvider).writeValueAsString(object);
    }

    public static long convertTypedIdToId(String typedId) {
        String[] pieces = typedId.split("\\.");
        return Long.parseLong(pieces[0]);
    }

    public static String convertTypedIdToTypeCode(String typedId) {
        return typedId.substring(typedId.indexOf('.') + 1);
    }

    private static class FilterOutIgnoredFieldsInstrospector extends JacksonAnnotationIntrospector {

        public static final String FILTER_ID = "FILTER_OUT_IGNORED_FIELDS";

        @Override
        public Object findFilterId(Annotated ac) {
            return FILTER_ID;
        }
    }

    public static TypeDescriptor getTypeDescriptorFor(VirtualFile file) {
        if (file == null) {
            return null;
        }

        VirtualFile parentLevel1 = file.getParent();
        if (parentLevel1 != null) {
            switch (parentLevel1.getName()) {
                case TransformProductExtension.DIRNAME:
                    return TransformProductExtension.DESCRIPTOR;
                case TransformCustomerExtension.DIRNAME:
                    return TransformCustomerExtension.DESCRIPTOR;
                case TransformPricingParameter.DIRNAME_BASE:
                    return TransformPricingParameter.DESCRIPTOR;
                case TransformCalculationLogic.DIRNAME:
                    return TransformCalculationLogic.DESCRIPTOR;
                case TransformWorkflowFormula.DIRNAME:
                    return TransformWorkflowFormula.DESCRIPTOR;
                case TransformPublishingTemplate.DIRNAME:
                    return TransformPublishingTemplate.DESCRIPTOR;
            }
            switch (file.getName()) {
                case TransformProductAttribute.DIRNAME:
                    return TransformProductAttribute.DESCRIPTOR;
                case TransformCustomerAttribute.DIRNAME:
                    return TransformCustomerAttribute.DESCRIPTOR;
                case TransformSellerAttribute.DIRNAME:
                    return TransformSellerAttribute.DESCRIPTOR;
                case TransformPublishingTemplate.DIRNAME:
                    return TransformPublishingTemplate.DESCRIPTOR;
                case TransformModelClass.DIRNAME:
                    return TransformModelClass.DESCRIPTOR;
            }
        }
        return null;
    }
}
